<?php

/*
Plugin Name: iVolunteer
Plugin URI: http://wordpress.org/extend/plugins/iVolunteer/
Description: Retrieves local community volunteer opportunities and displays them in the sidebar. After activating the Plugin, go to Widgets to add iVolunteer to your sidebar.
Version: 1.0.4
Author: Neil Simon
Author URI: http://actionfeed.org/
*/


/*
 Copyright 2009 iVolunteer.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110 USA
*/


// From wordpress271/js/tinymce/plugins/spellchecker/classes/utils
include_once ('JSON.php');


// Constants
define ('IVOLUNTEER_PLUGIN_VERSION',   'iVolunteer-v1.0.4');
define ('IVOLUNTEER_SETTINGS_OPTIONS', 'iVolunteer_settingsOptions');
define ('IVOLUNTEER_API_URL',          'http://actionfeed.org/server/resources/events/consolidated');
define ('IVOLUNTEER_URL',              'http://actionfeed.org/');


function iVolunteer_cURL ()
    {
    // Init curl session
    $hCurl = curl_init ();

    // Return response from curl_exec() into variable
    curl_setopt ($hCurl, CURLOPT_RETURNTRANSFER, 1);

    // Get the visitor's ip address
    if (($ip = $_SERVER ['REMOTE_ADDR']) == "127.0.0.1")
        {
        // Use this for testing on //localhost
        $ip = "67.166.52.44";
        }

    // Set the URL
    curl_setopt ($hCurl, CURLOPT_URL, 'http://actionfeed.org/server/resources/events/consolidated/?ip=' . $ip);

    // Exec the call
    $response = curl_exec ($hCurl);

    // Close the session
    curl_close ($hCurl);

    // Output the results
    return ($response);
    }


function iVolunteer_buildSidebar (&$buf)
    {
    $rc = 1;  // Reset to 0 on success

    // Get the iVolunteer local events list -- returned as XML
    if (($response = iVolunteer_cURL ()) == FALSE)
        {
        // This can happen when the site is found, but the API is down
        // -- just exit
        }

    elseif (strpos ($response, "Not Found"))
        {
        // This can happen when the site is NOT found
        // -- just exit
        }

    else
        {
        // Instantiate JSON object
        $json_iVolunteer = new Moxiecode_JSON ();

        // Decode the returned JSON
        $consolidated = $json_iVolunteer->decode ($response);

        // Parse JSON and load sidebar buf
        parse_json ($consolidated, $buf);

        // NULL the object
        $json_iVolunteer = 0;

        $rc = 0;
        }

    return ($rc);
    }


function parse_json ($consolidated, &$buf)
    {
    $organizations = $consolidated ['organizations'];
    $timestamps    = $consolidated ['timestamps'];  // iso 8601 format
    $events        = $consolidated ['events'];
    $locations     = $consolidated ['locations'];

    $locationsNormalized = array ();
    foreach ($locations as $location) 
        {
        $locationsNormalized [$location ['id']] = $location;
        }

    $timestampsNormalized = array ();
    foreach ($timestamps as $timestamp) 
        {
        $timestampsNormalized [$timestamp ['id']] = $timestamp;
        }

    $buf .= 'Upcoming Volunteer Opportunities:<br /><br />';

    // Open list
    $buf .= '<ul>';

    $maxEvents = 5;
    $eventCount = 0;

    foreach ($events as $event)
        {
        if ($eventCount++ < $maxEvents)
            {
            $organizationIds = (array) $event ['organizationCollection'];
            $timestampIds    = (array) $event ['timestampCollection'];
            $locationIds     = (array) $event ['locationCollection'];

            $event ['organizations'] = array ();
            $event ['timestamps']    = array ();
            $event ['locations']     = array ();

            foreach ($organizationIds as $organizationId)
                {
                $event ['organizations'] [$organizationId] = $organizations [$organizationId];
                }

            foreach ($timestampIds as $timestampId)
                {
                $event ['timestamp'] = $timestampsNormalized [$timestampId];
                }

            foreach ($locationIds as $locationId)
                {
                $event ['location'] = $locationsNormalized [$locationId];
                }

            $eventTitle    = $event ['title'];
            $eventTime     = $event ['timestamp'] ['timestamp'];
            $eventLocation = $event ['location'] ['city']  . ', ' .
                             $event ['location'] ['state'] . ', ' .
                             $event ['location'] ['zip'];
            $eventPhone    = $event ['phone'];
            $eventUrl      = $event ['url'];

            // Open the list item
            $buf .= '<li>';

            // Remove non-ascii characters from $eventTitle
            $eventTitleCleaned = preg_replace ('/[^(\x20-\x7F)]*/', '', $eventTitle);

            // Display the event title as a clickable link
            $buf .= '<a href="' . $eventUrl . '" target="_blank">' . $eventTitleCleaned . '</a><br />';

            // Split-out the date (ex. 2009-05-29)
            $datePart = substr ($eventTime, 0, 10);

            // Split-out the time (ex. 08:00)
            $timePart = substr ($eventTime, 11, 5);

            // Display the date -- but stay on the same line
            $buf .= $datePart;

            // If the time is NOT 00:00...
            if ($timePart != "00:00")
                {
                // Add the time to the date line
                $buf .= ' &nbsp; ' . $timePart;
                }

            // Line break
            $buf .= '<br />';

            // Display the location line (city, state, zip)
            $buf .= $eventLocation    . '<br />';

            // If the phone number is NOT blank...
            if ($eventPhone != "")
                {
                // Display it
                $buf .= $eventPhone    . '<br />';
                }

            // Close the list item
            $buf .= '<br /></li>';
            }
        }

    // Close list
    $buf .= '</ul>';
    }


function iVolunteer_initWidget ()
    {
    // MUST be able to register the widget... else exit
    if (function_exists ('register_sidebar_widget'))
        {
        // Declare function -- called from Wordpress -- during page-loads
        function iVolunteer_widget ($args)
            {
            // Load existing options from wp database
            $iVolunteer_settingsOptions = get_option (IVOLUNTEER_SETTINGS_OPTIONS);

            // Accept param array passed-in from WP (e.g. $before_widget, $before_title, etc.), theme CSS styles
            extract ($args);

            // Display sidebar title above the about-to-be-rendered iVolunteer events table
            echo $before_widget . $before_title .
                 '<a href="http://actionfeed.org" title="Visit iVolunteer" target="_blank">iVolunteer</a>' .
                 $after_title;

            // Open a plugin-version-tracking DIV tag
            printf ("<div id=\"%s\">", IVOLUNTEER_PLUGIN_VERSION);

            $buf = '';

            // Dynamically build the table and display it
            if (iVolunteer_buildSidebar ($buf) == 0)
                {
                printf ("%s", $buf);
                }

            // Close the plugin-version-tracking DIV tag
            printf ("</div>");

            echo $after_widget;
            }

        // Register the widget function to be called from Wordpress on each page-load
        register_sidebar_widget ('iVolunteer', 'iVolunteer_widget');
        }
    }


function iVolunteer_createOptions ()
    {
    // Create the initialSettingsOptions array of keys/values
    $iVolunteer_initialSettingsOptions = array ('numEventsToDisplay' => 'iVolunteer');

    // Store the initial options to the wp database
    add_option (IVOLUNTEER_SETTINGS_OPTIONS, $iVolunteer_initialSettingsOptions);
    }


function iVolunteer_deleteOptions ()
    {
    // Remove the iVolunteer_settingsOptions array from the wp database
    delete_option (IVOLUNTEER_SETTINGS_OPTIONS);
    }


function iVolunteer_updateSettingsPage ()
    {
    // Load existing options from wp database
    $iVolunteer_settingsOptions = get_option (IVOLUNTEER_SETTINGS_OPTIONS);

    // Localize all displayed strings
    $iVolunteer_enterOptionsStr       = __('Please enter the iVolunteer Plugin options:',   'ivolunteer');
    $iVolunteer_numEventsToDisplayStr = __('Number of events to display:',                  'ivolunteer');
    $iVolunteer_saveStr               = __('Save',                                          'ivolunteer');
    $iVolunteer_optionsSavedStr       = __('iVolunteer Plugin options saved successfully.', 'ivolunteer');

    // If ALL data fields contain values...
    if (isset ($_POST ['numEventsToDisplay']))
        {
        // Copy the fields to the persistent wp options array
        $iVolunteer_settingsOptions ['numEventsToDisplay'] = $_POST ['numEventsToDisplay'];

        // Store changed options back to wp database
        update_option (IVOLUNTEER_SETTINGS_OPTIONS, $iVolunteer_settingsOptions);

        // Display update message to user
        echo '<div id="message" class="updated fade"><p>' . $iVolunteer_optionsSavedStr . '</p></div>';
        }

    // Display form to the user

    echo
     '<div class="wrap">

      <h3>&nbsp;' . $iVolunteer_enterOptionsStr . '</h3>

      <p>NOTE: Only 5 opportunities are currently displayed. The ability to choose more (or fewer) will be activated soon.</p>

      <form action="" method="post">

      <table border="0" cellpadding="10">

      <tr>
      <td>' . $iVolunteer_numEventsToDisplayStr . ' &nbsp;</td>
      <td><input type="text" name="numEventsToDisplay" value="' . $iVolunteer_settingsOptions ['numEventsToDisplay'] . '" size="40" /></td>
      </tr>

      </table>

      <br />

      <p>&nbsp;<input type="submit" value="' . $iVolunteer_saveStr . '" /></p>

      </form>

      </div>';
    }


function iVolunteer_addSubmenu ()
    {
    // Define the options for the submenu page
    add_submenu_page ('options-general.php',             // Parent page
                      'iVolunteer page',                 // Page title, shown in titlebar
                      'iVolunteer',                      // Menu title
                      10,                                // Access level all
                      __FILE__,                          // This file displays the options page
                      'iVolunteer_updateSettingsPage');  // Function that displays options page
    }


// Initialize for localized strings
//
// If (the WordPress config file "wp-config.php" has a language defined, e.g. ('WPLANG', 'it_IT'))
//     {
//     WordPress will look in the plugin directory for a corresponding localized strings file and load it
//     (e.g. "ivolunteer-it_IT.mo")
//     }
load_plugin_textdomain ('ivolunteer', 'wp-content/plugins/ivolunteer');


// Runs only once at activation time
register_activation_hook (__FILE__, 'iVolunteer_createOptions');


// Runs only once at deactivation time
register_deactivation_hook (__FILE__, 'iVolunteer_deleteOptions');


// Load the widget, show it in the widget control in the admin section
add_action ('plugins_loaded', 'iVolunteer_initWidget');


// Add the iVolunteer submenu to the Settings page
add_action ('admin_menu', 'iVolunteer_addSubmenu');

?>
